@@ -7,32 +7,37 @@ heroku3.api
 This module provides the basic API interface for Heroku.
 """
 
+import sys
+from pprint import pprint  # noqa
+
+import requests
+from requests.exceptions import HTTPError
+
 from .compat import json
 from .helpers import is_collection
 from .models import Plan, RateLimit
-from .models.app import App
-from .models.addon import Addon
-from .models.dyno import Dyno
 from .models.account import Account
-from .models.key import Key
+from .models.account.feature import AccountFeature
+from .models.addon import Addon
+from .models.app import App
 from .models.configvars import ConfigVars
+from .models.dyno import Dyno
+from .models.invoice import Invoice
+from .models.key import Key
 from .models.logsession import LogSession
-from .models.oauth import OAuthClient, OAuthAuthorization, OAuthToken
+from .models.oauth import OAuthAuthorization, OAuthClient, OAuthToken
 from .rendezvous import Rendezvous
 from .structures import KeyedListResource, SSHKeyListResource
-from .models.account.feature import AccountFeature
-from requests.exceptions import HTTPError
-from pprint import pprint # noqa
-import requests
-import sys
 
 if sys.version_info > (3, 0):
     from urllib.parse import quote
 else:
-    from urllib import quote # noqa
+    from urllib import quote  # noqa
 
 
 HEROKU_URL = 'https://api.heroku.com'
+HEROKU_HEADERS = {'Accept': 'application/vnd.heroku+json; version=3.cedar-acm',
+                  'Content-Type': 'application/json'}
 
 
 class RateLimitExceeded(Exception):
@@ -59,8 +64,7 @@ class HerokuCore(object):
         self._last_request_id = None
 
         # We only want JSON back.
-        #self._session.headers.update({'Accept': 'application/json'})
-        self._session.headers.update({'Accept': 'application/vnd.heroku+json; version=3', 'Content-Type': 'application/json'})
+        self._session.headers.update(HEROKU_HEADERS)
 
     def __repr__(self):
         return '<heroku-core at 0x%x>' % (id(self))
@@ -106,36 +110,65 @@ class HerokuCore(object):
         except ValueError:
             raise ResponseError('The API Response was not valid.')
 
-    def _get_headers_for_request(self, method, url, legacy=False, order_by=None, limit=None, valrange=None, sort=None):
+    def _get_headers_for_request(self,
+                                 legacy=False,
+                                 order_by=None,
+                                 limit=None,
+                                 valrange=None,
+                                 sort=None,
+                                 ):
         headers = {}
         if legacy is True:
-            #Nasty patch session to fallback to old api
+            # Nasty patch session to fallback to old api
             headers.update({'Accept': 'application/json'})
 
         else:
             range_str = None
+            # "Range: name ..; order=desc,max=10;"#
             if order_by or limit or valrange or sort:
                 range_str = ""
+                seperator = ''
                 if order_by:
-                    range_str = "{0} ..".format(order_by)
+                    range_str = "{0} ..;".format(order_by)
+                else:
+                    range_str = "id ..;"
+
+                if sort is not None:
+                    assert(sort == 'asc' or sort == 'desc')
+                    range_str += " order={0}".format(sort)
+                    seperator = ','
+                else:
+                    range_str += ' '
+
                 if limit:
                     if limit > 1000:
-                        raise MaxRangeExceeded("Your *limit* ({0}) argument is greater than the maximum allowed value of 1000".format(limit))
-                    range_str += "; max={0}".format(limit)
-                if not sort is None:
-                    assert(sort == 'asc' or sort == 'desc')
-                    range_str += "; order={0}".format(sort)
+                        raise MaxRangeExceeded("Your *limit* ({0}) argument is"
+                                               " greater than the maximum "
+                                               "allowed value of 1000"
+                                               .format(limit))
+                    range_str += "{0}max={1}".format(seperator, limit)
 
+                range_str += ';'
+                # print(range_str)
                 if valrange:
-                    #If given, This should override limit and order_by
+                    # If given, This should override limit and order_by
                     range_str = valrange
 
-            if not range_str == None:
+            if range_str is not None:
                 headers.update({'Range': range_str})
 
         return headers
 
-    def _http_resource(self, method, resource, params=None, data=None, legacy=False, order_by=None, limit=None, valrange=None, sort=None):
+    def _http_resource(self,
+                       method,
+                       resource,
+                       params=None,
+                       data=None,
+                       legacy=False,
+                       order_by=None,
+                       limit=None,
+                       valrange=None,
+                       sort=None):
         """Makes an HTTP request."""
 
         if not is_collection(resource):
@@ -143,10 +176,12 @@ class HerokuCore(object):
 
         url = self._url_for(*resource)
 
-        headers = self._get_headers_for_request(method, url, legacy=legacy, order_by=order_by, limit=limit, valrange=valrange, sort=sort)
+        headers = self._get_headers_for_request(legacy=legacy,
+                                                order_by=order_by,
+                                                limit=limit,
+                                                valrange=valrange,
+                                                sort=sort)
 
-        #print "\n\n\n\n"
-        #print url
         r = self._session.request(method, url, params=params, data=data, headers=headers)
 
         if 'ratelimit-remaining' in r.headers:
@@ -155,8 +190,8 @@ class HerokuCore(object):
         if 'Request-Id' in r.headers:
             self._last_request_id = r.headers['Request-Id']
 
-        #if 'Accept-Ranges' in r.headers:
-            #print "Accept-Ranges = {0}".format(r.headers['Accept-Ranges'])
+        # if 'Accept-Ranges' in r.headers:
+        #     print("Accept-Ranges = {0}".format(r.headers['Accept-Ranges']))
 
         if r.status_code == 422:
             http_error = HTTPError('%s - %s Client Error: %s' %
@@ -165,18 +200,12 @@ class HerokuCore(object):
             raise http_error
 
         if r.status_code == 429:
-            #Rate limit reached
-            print r.headers
+            # Rate limit reached
             raise RateLimitExceeded("You have exceeded your rate limit \n{0}".format(r.content.decode("utf-8")))
 
         if (not str(r.status_code).startswith('2')) and (not r.status_code in [304]):
-            print "Status not sensible - {0}".format(r.status_code)
-            print r.headers
-            print r.content.decode("utf-8")
             pass
         r.raise_for_status()
-        #print r.content.decode("utf-8")
-        #print "\n\n\n\n"
         return r
 
     def _get_resource(self, resource, obj, params=None, **kwargs):
@@ -202,9 +231,9 @@ class HerokuCore(object):
 
         items = self._resource_deserialize(r.content.decode("utf-8"))
         if r.status_code == 206 and 'Next-Range' in r.headers and not limit:
-            #We have unexpected chunked response - deal with it
+            # We have unexpected chunked response - deal with it
             valrange = r.headers['Next-Range']
-            print "Warning Response was chunked, Loading the next Chunk using the following next-range header returned by Heroku '{0}'. WARNING - This breaks randomly depending on your order_by name. I think it's only guarenteed to work with id's - Looks to be a Heroku problem".format(valrange)
+            print("Warning Response was chunked, Loading the next Chunk using the following next-range header returned by Heroku '{0}'. WARNING - This breaks randomly depending on your order_by name. I think it's only guarenteed to work with id's - Looks to be a Heroku problem".format(valrange))
             new_items = self._get_data(resource, params=params, legacy=legacy, order_by=order_by, limit=limit, valrange=valrange, sort=sort)
             items.extend(new_items)
 
@@ -213,9 +242,9 @@ class HerokuCore(object):
     def _process_items(self, d_items, obj, map=None, **kwargs):
 
         if not isinstance(d_items, list):
-            print "Warning, Response for '{0}' was of type {1} - I was expecting a 'list'. This could mean the api has changed its response type for this request.".format(obj, type(d_items))
+            print("Warning, Response for '{0}' was of type {1} - I was expecting a 'list'. This could mean the api has changed its response type for this request.".format(obj, type(d_items)))
             if isinstance(d_items, dict):
-                print "As it's a dict, I'll try to process it anyway"
+                print("As it's a dict, I'll try to process it anyway")
                 return self._process_item(d_items, obj, **kwargs)
 
         items = [obj.new_from_dict(item, h=self, **kwargs) for item in d_items]
@@ -258,11 +287,17 @@ class Heroku(HerokuCore):
     def app(self, id_or_name):
         return self._get_resource(('apps/{0:s}'.format(id_or_name)), App)
 
-    def create_app(self, name=None, stack_id_or_name='cedar', region_id_or_name=None):
+    def create_app(self, name=None, stack_id_or_name='cedar', region_id_or_name=None, organization=None):
         """Creates a new app."""
 
         payload = {}
 
+        if organization:
+            payload['organization'] = organization
+            resource = ('organizations', 'apps')
+        else:
+            resource = ('apps',)
+
         if name:
             payload['name'] = name
 
@@ -275,7 +310,7 @@ class Heroku(HerokuCore):
         try:
             r = self._http_resource(
                 method='POST',
-                resource=('apps',),
+                resource=resource,
                 data=self._resource_serialize(payload)
             )
             r.raise_for_status()
@@ -283,15 +318,21 @@ class Heroku(HerokuCore):
             app = App.new_from_dict(item, h=self)
         except HTTPError as e:
             if "Name is already taken" in str(e):
-                print "Warning - {0:s}".format(e)
-                app = self.app(name)
-                pass
+                try:
+                    app = self.app(name)
+                except:
+                    raise
+                else:
+                    print("Warning - {0:s}".format(e))
             else:
-                raise e
+                raise
         return app
 
     def keys(self, **kwargs):
-        return self._get_resources(('user', 'keys'), Key, map=SSHKeyListResource, **kwargs)
+        return self._get_resources(('account/keys'), Key, map=SSHKeyListResource, **kwargs)
+
+    def invoices(self,**kwargs):
+        return self._get_resources(('account/invoices'),Invoice)
 
     def labs(self, **kwargs):
         return self.features(**kwargs)
