@@ -1,8 +1,12 @@
-from ..models import BaseResource, User, Stack
+import sys
+from pprint import pprint  # NOQA
+
+from heroku3.models.slug import Slug
+from ..models import BaseResource, Stack, User, Organization
 from ..rendezvous import Rendezvous
 from ..structures import DynoListResource
-
 from .addon import Addon
+from .build import Build
 from .collaborator import Collaborator
 from .configvars import ConfigVars
 from .domain import Domain
@@ -13,9 +17,6 @@ from .logsession import LogSession
 from .region import Region
 from .release import Release
 
-from pprint import pprint # NOQA
-import sys
-
 if sys.version_info > (3, 0):
     from urllib.parse import quote
 else:
@@ -29,7 +30,7 @@ class App(BaseResource):
     _ints = ['slug_size', 'repo_size']
     _bools = ['maintenance']
     _dates = ['archived_at', 'created_at', 'released_at', 'updated_at']
-    _map = {'region': Region, 'owner': User, 'stack': Stack}
+    _map = {'region': Region, 'owner': User, 'stack': Stack, 'organization': Organization}
     _pks = ['name', 'id']
 
     def __init__(self):
@@ -47,6 +48,15 @@ class App(BaseResource):
             obj=Addon, app=self, **kwargs
         )
 
+    def builds(self, **kwargs):
+        """
+        Returns a list of application builds as Build objects
+        """
+        return self._h._get_resources(
+            resource=('apps', self.name, 'builds'),
+            obj=Build, app=self, **kwargs
+        )
+
     def delete(self):
         """
         Destroys the current app
@@ -148,6 +158,17 @@ class App(BaseResource):
         r.raise_for_status()
         item = self._h._resource_deserialize(r.content.decode("utf-8"))
         return ConfigVars.new_from_dict(item, h=self._h, app=self)
+    
+    def get_domain(self, hostname_or_id):
+        """Get the domain for this app.."""
+        r = self._h._http_resource(
+            method='GET',
+            resource=('apps', self.name, 'domains', hostname_or_id),
+        )
+        r.raise_for_status()
+        
+        item = self._h._resource_deserialize(resp.content.decode("utf-8"))
+        return Domain.new_from_dict(item, h=self._h, app=self)
 
     def domains(self, **kwargs):
         """The domains for this app."""
@@ -201,11 +222,11 @@ class App(BaseResource):
                 dyno.kill()
         return self
 
-    def run_command_detached(self, command, size=1, env=None):
+    def run_command_detached(self, command, size=None, env=None):
         """Run a remote command but do not wait for the command to complete"""
         return self.run_command(command, attach=False, printout=False, size=size, env=env)
 
-    def run_command(self, command, attach=True, printout=True, size=1, env=None):
+    def run_command(self, command, attach=True, printout=True, size=None, env=None):
         """Run a remote command attach=True if you want to capture the output"""
         if attach:
             attach = True
@@ -214,6 +235,9 @@ class App(BaseResource):
         if env:
             payload['env'] = env
 
+        if size:
+            payload['size'] = size
+
         r = self._h._http_resource(
             method='POST',
             resource=('apps', self.name, 'dynos'),
@@ -237,6 +261,50 @@ class App(BaseResource):
             obj=Formation, app=self, **kwargs
         )
 
+    def batch_scale_formation_processes(self, updates):
+        """Scale formation processes in batch.
+
+        updates is a dictionary or iterable of 2-tuples of
+        formation_id_or_name to quantity.
+        """
+        payload = {}
+        payload['updates'] = []
+
+        for type, quantity in dict(updates).items():
+            assert(quantity == 0 or quantity)
+            payload['updates'].append({'type': type, 'quantity': quantity})
+
+        r = self._h._http_resource(
+            method='PATCH',
+            resource=('apps', self.id, 'formation'),
+            data=self._h._resource_serialize(payload)
+        )
+        r.raise_for_status()
+        decoded = self._h._resource_deserialize(r.content.decode("utf-8"))
+        return self._h._process_items(decoded, obj=Formation)
+
+    def batch_resize_formation_processes(self, updates):
+        """Resize formation processes in batch.
+
+        updates is a dictionary or iterable of 2-tuples of
+        formation_id_or_name to size.
+        """
+        payload = {}
+        payload['updates'] = []
+
+        for type, size in dict(updates).items():
+            assert(size == 0 or size)
+            payload['updates'].append({'type': type, 'size': size})
+
+        r = self._h._http_resource(
+            method='PATCH',
+            resource=('apps', self.id, 'formation'),
+            data=self._h._resource_serialize(payload)
+        )
+        r.raise_for_status()
+        decoded = self._h._resource_deserialize(r.content.decode("utf-8"))
+        return self._h._process_items(decoded, obj=Formation)
+
     def scale_formation_process(self, formation_id_or_name, quantity):
         assert(quantity == 0 or quantity)
         payload = {}
@@ -436,6 +504,18 @@ class App(BaseResource):
 
         return LogSession.new_from_dict(item, h=self._h, app=self)
 
+    def create_release(self, slug_id):
+       """Create a new release for this app."""
+       payload = {'slug': slug_id}
+       r = self._h._http_resource(
+           method='POST',
+           resource=('apps', self.name, 'releases'),
+           data=self._h._resource_serialize(payload),
+       )
+       r.raise_for_status()
+       item = self._h._resource_deserialize(r.content.decode("utf-8"))
+       return Release.new_from_dict(item, h=self._h, app=self)
+
     def releases(self, **kwargs):
         """The releases for this app."""
         return self._h._get_resources(
@@ -454,6 +534,28 @@ class App(BaseResource):
         r.raise_for_status()
         return self.releases()[-1]
 
+    def update_buildpacks(self, buildpack_urls):
+        payload = {
+            'updates': [
+                {'buildpack': url} for url in buildpack_urls
+            ]
+        }
+
+        r = self._h._http_resource(
+            method='PUT',
+            resource=('apps', self.id, 'buildpack-installations'),
+            data=self._h._resource_serialize(payload)
+        )
+        r.raise_for_status()
+        return r.ok
+
+    def slug(self, slug_id):
+        """Get a slug by id."""
+        return self._h._get_resources(
+            resource=('apps', self.name, 'slugs', slug_id),
+            obj=Slug, app=self,
+        )
+
 
 class AppTransfer(BaseResource):
     _strs = ['id', 'state']
