@@ -1,10 +1,10 @@
-from .  import BaseResource
+from . import BaseResource
 
 
 class Key(BaseResource):
     """Heroku SSH Key."""
 
-    _strs = ['id', 'public_key', 'email', 'fingerprint']
+    _strs = ['id', 'public_key', 'email', 'fingerprint', 'comment']
     _dates = ['created_at', 'updated_at']
     _pks = ['id']
 
