@@ -1,9 +1,9 @@
-from .  import BaseResource
+from . import BaseResource
 from .addon import Addon
 
 
 class LogDrain(BaseResource):
-    _strs = ['id', 'url']
+    _strs = ['id', 'url', 'token']
     _map = {'addon': Addon}
     _dates = ['created_at', 'updated_at']
     _pks = ['id']
